﻿/*
Copyright (C) 2007 Olga Khylkouskaya

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/ 

/**
 * @fileoverview cruisecontrol viewer gadget
 * 
 * view the status of projects in cruisecontrol
 */ 

var cruiseControlUrl;
/**
 * list of projects to check status
 * @type Array
 */ 
var projects = new Array();

/**
 * map projectName to AjaxHelper
 * @type Array
 */ 
var project2Helper = new Array();
/**
 * map projectName to update status timeout id which we get from setTimeout function
 * @type Array
 */ 
var project2TimeoutId = new Array();

// represent one row in gadget table: project link, status image, updated info
var xmlLabelProj = "<a height='16' width='92' x='0' y='interval' href='cruiseControlUrl/common'>common</a>";
var imgProj = "<img height='16' name='commonImg' width='16' x='103' y='interval' src='images/white.png'/>";
var statusProj = "<label height='16' name='commonStatus' width='67' x='140' y='interval' size='9'>test</label>";
/**
 * interval between rows
 * @type Number
 */ 
var interval = 20;

var refreshTimeout = 3000;
var isInit = false;
var isDock = true;

function view_onOpen() {
  if (!isInit && !options.getValue("cruiseControlEditProp"))  {//if it is not null than we have initialized it
	var withoutOptions = "<label name='noOptions' height='50' width='200' x='0' y='10' size='14' align='center'>" + SET_OPTIONS_TEXT + "</label>";
	var optLink = "<a height='100' x='80' y='60' onclick='onOpenOptionsClick();' font='11' align='center'>" + OPTIONS_TEXT + "</a>";
	contentDiv.appendElement(withoutOptions);
	contentDiv.appendElement(optLink);
  }
  else {
    recreateProjectList();
  }
}

/**
 * delete all previous elements, create elements according to projects array
 * create row(project link, status image, updated info) for each project
 * 
 */ 
function createElements() {
  contentDiv.removeAllElements();
  var newHeight = interval * projects.length;
  if (newHeight < interval) {
    newHeight = interval;
  }
  
  var addedHeight = newHeight - contentDiv.height;
  contentDiv.height = newHeight;
  changeViewSize(addedHeight);

  for (var i=0;i<projects.length;++i) {
    var label = xmlLabelProj.replace(/common/g, projects[i]).replace(/interval/g, (i*interval) + "");
    label = label.replace(/cruiseControlUrl/g, cruiseControlUrl);
    contentDiv.appendElement(label);
    var img = imgProj.replace(/common/g, projects[i]).replace(/interval/g, (i*interval) + "");
    contentDiv.appendElement(img);
    var status = statusProj.replace(/common/g, projects[i]).replace(/interval/g, (i*interval + 1) + "");
    contentDiv.appendElement(status);
  }
}

/**
 * run process of updating status
 * 
 */ 
function runProcess() {
  //kill previous process
  killProcess();
  //clear status text
  for (i in projects) {
    if (contentDiv.children.item(projects[i] + "Status")) {
	contentDiv.children.item(projects[i] + "Status").innerText = "";
    }
  }

  for (i in projects) {
    updateStatus(cruiseControlUrl, projects[i]);
  }
}
/**
 * send request to url(url+projectName) using ajax
 * 
 * @param {String} url The url to cruisecontrol dashboard
 * @param {String} projectName Project name to access status of this project in
 * cruise control
 */ 
function updateStatus(url, projectName) {
  if (contentDiv.children.item(projectName + "Status")) {
    contentDiv.children.item(projectName + "Status").innerText=STATUS_UPDATING;
    var helper;
    if (!project2Helper[projectName]) {
      project2Helper[projectName] = new AjaxHelper();
    }
    helper = project2Helper[projectName];

    helper.sendUrl(url, projectName, success, failed);
  }
}

/**
 * stop updating process: clear timeout and abort ajax request
 * 
 */ 
function killProcess() {
  for (var proj in project2TimeoutId) { //clear timeout
    clearTimeout(project2TimeoutId[proj]);
  }
  for (var projName in project2Helper) { //abort sending
    project2Helper[projName].abortProcess();
  }
}
/**
 * clear previous timeout
 * 
 * @param {String} projectName Name of project which timeout we need to kill
 */ 
function killPreviousTimeout(projectName) {
  if (project2TimeoutId[projectName]) {
    clearTimeout(project2TimeoutId[projectName]);
  }
}

/**
 * is called after ajax request is successfull
 * update status and set timeout to the next update
 */ 
function success(responseText, projectName) {
  if (contentDiv.children.item(projectName + "Status")) {
    contentDiv.children.item(projectName + "Status").innerText = STATUS_UPDATED;
    if (responseText.indexOf(FIND_TEXT_BUILD_COMPLETE) != -1) {
      contentDiv.children.item(projectName + "Img").src="images/green.png";
    }
    else if (responseText.indexOf(FIND_TEXT_BUILD_FAILED) != -1) {
      contentDiv.children.item(projectName + "Img").src="images/red.png";
    }
    else {
      debug.trace("yellow to project: " + projectName);
      contentDiv.children.item(projectName + "Img").src="images/yellow.png";
    }

    updateLabel.innerText = Util.getNowTimeFormatting(); 

    killPreviousTimeout(projectName);
    project2TimeoutId[projectName] = setTimeout("updateStatus('" + cruiseControlUrl + "', '" + projectName + "')", refreshTimeout);
  }
}

/**
 * is called after ajax request is failed
 * 
 * @param {String} projectName The name of project
 */ 
function failed(projectName) {
  if (contentDiv.children.item(projectName + "Status")) {
    contentDiv.children.item(projectName + "Status").innerText = STATUS_FAILED;
    contentDiv.children.item(projectName + "Img").src="images/white.png";

    killPreviousTimeout(projectName);
    project2TimeoutId[projectName] = setTimeout("updateStatus('" + cruiseControlUrl + "', '" + projectName + "')", refreshTimeout);
  }
}

/**
 * when some option changed update information and refresh the process
 */ 
function onOptionChanged() {
  // optionChanged will contain the name of the option which was changed
  var optionChanged = event.propertyName;	
  // Update only the appropriate property
  if (optionChanged == "listProjectsProp") {
    text = options.getValue("listProjectsProp");
    var projectsArray = Util.convertToArray(text);
    if (Util.isChangedArray(projects, projectsArray)) {
      view.alert(PROJ_LIST_CHANGED);
      recreateProjectList();
    }
  } else if (optionChanged == "intervalEditProp") {
     var intervalStr = options.getValue("intervalEditProp");
     var number = parseInt(intervalStr);
     refreshTimeout = number*1000;
        
     runProcess();//refresh
  } else if (optionChanged == "cruiseControlEditProp") {
      cruiseControlUrl = options.getValue("cruiseControlEditProp");

      if (!isInit) {
	recreateProjectList();
      }
      else {
        runProcess();//refresh
      }
  }
}
/**
 * remove all project names in projects list when you change
 * projects names in options dialog
 * 
 */ 
function recreateProjectList() {
  //kill it before create new elements
  killProcess();
  isInit = true;

  text = options.getValue("listProjectsProp");
  var projectsArray = Util.convertToArray(text);

  projects = Util.normalizeArr(projectsArray);
  project2Helper = new Array();//clear it so
  project2TimeoutId = new Array();

  //when we minimize we need to reread it
  refreshTimeout = parseInt(options.getValue("intervalEditProp"))*1000;
  cruiseControlUrl = options.getValue("cruiseControlEditProp");

  createElements();
  runProcess();
}
/**
 * change size of view
 * 
 * @param {Number} val on how much to change size, can be negative
 */ 
function changeViewSize(val) {
    var zoomHeight = view.height;
    view.resizable = 'true';
    var normalHeight = view.height;
    var koeff = zoomHeight/normalHeight;
    view.resizable = 'zoom';
    //view.alert("koeff: " + koeff);
  //view.height = view.height + 20;
  //if (normalHeight + val >= minViewHeight) {
    var newViewHeight = view.height + val*koeff;
    //view.alert("new view height: " + newViewHeight);
		view.height = newViewHeight
	  //change images height
    backImg.height = newViewHeight;
    bottomImg.y = newViewHeight - bottomImg.height;
  //}
  
}
function onOpenOptionsClick() {
  pluginHelper.ShowOptionsDialog();
} 

function pluginHelper.onAddCustomMenuItems(menu) 
{
	menu.AddItem(MEN_REFRESH, 0, runProcess);
}
